'use strict';

let choices;

module.exports = {

  validateBranch() {
    this.params.actualbranch = this.params.actualbranch ? this.params.actualbranch : this.scmGetBranch();
    const branches = this.params.startBranches[this.params.actualbranch];
    if (branches && Object.getOwnPropertyNames(branches).indexOf(this.params.targetBranch) >= 0) {
      return Promise.resolve();
    } else {
      return Promise.reject({error: `A branch called "${this.params.targetBranch}" can not be started from a "${this.params.actualbranch}" branch!`});
    }
  },

  _askContinue(others) {
    this.params.promptGoon = [
      {
        type: 'confirm',
        name: '_goon',
        required: true,
        message: `There are one or more ${others} branches. Do you want to continue ?`
      }
    ];
    return this.inquire('promptGoon').then(() => {
      if (!this.params._goon) {
        throw {error: 'User stops execution...'};
      }
    });
  },

  _checkOtherBranches() {
    this.params.actualbranch = this.params.actualbranch ? this.params.actualbranch : this.scmGetBranch();
    const config = this.params.startBranches[this.params.actualbranch];
    if (config && config[this.params.targetBranch] && config[this.params.targetBranch].checkOthers) {
      return this.execute(this.gitCmd, ['branch', '--all'])
        .then((result) => {
          if (result && result.output) {
            let n = 0;
            config[this.params.targetBranch].checkOthers.forEach((branch) => result.output.toString().indexOf(branch) >= 0 ? n++ : null);
            if (n > 0) {
              return this._askContinue(config[this.params.targetBranch].checkOthers);
            }
          }
        });
    } else {
      return Promise.resolve();
    }
  },

  _changeVersionOrigin() {
    const branches = this.params.startBranches[this.scmGetBranch()];
    const options = branches[this.params.targetBranch];
    if (options && options.originVersionChange) {
      return this.semverConfig(options.originVersionChange, true)
        .then((config) => this.semverStrategy(config))
        .then((config) => this.semverSaveVersion(config));
    } else {
      return Promise.resolve();
    }
  },

  getChoices() {
    if (choices === undefined) {
      choices = [];
      this.params.actualbranch = this.params.actualbranch ? this.params.actualbranch : this.scmGetBranch();
      const branches = this.params.startBranches[this.params.actualbranch];
      if (branches) {
        Object.getOwnPropertyNames(branches).forEach((name) => {
          choices.push({value: name, name: branches[name].title});
        });
      } else {
        throw {error: `There is no target configured for ${this.params.actualbranch} branch`};
      }
    }
    return choices;
  },
  areChoices() {
    choices = this.getChoices();
    if (choices.length === 1) {
      this.params.targetBranch = choices[0].value;
    }
    return choices.length > 1;
  },

  getBranchName() {
    this.params.promptName = [
      {
        type: 'input',
        name: 'branchName',
        required: true,
        message: `Write the name of the new ${this.params.targetBranch} (${this.params.targetBranch}/ is already appended)`
      }
    ];
    return this.inquire('promptName').then(() => this.params.branchName = `${this.params.targetBranch}/${this.params.branchName}`);
  },

  // ---- STAGES ----

  check(ok, ko) {
    return this.scmFetch()
      .then(ok, ko);
  },

  run(ok, ko) {
    return this._checkOtherBranches()
      .then(() => this.validateBranch())
      .then(() => this.getBranchName())
      .then(() => this.execute(this.gitCmd, ['branch', this.params.branchName]))
      .then(() => this._changeVersionOrigin())
      .then(() => this.execute(this.gitCmd, ['checkout', this.params.branchName]))
      .then(ok, ko);
  },

  emit() {
    return {
      branchName: this.params.branchName
    };
  }
};